@@ -206,9 +206,9 @@ void hbf_free_transfer( hbf_transfer_t *transfer ) {
 
     for( cnt = 0; cnt < transfer->block_cnt; cnt++ ) {
         hbf_block_t *block = transfer->block_arr[cnt];
+        if( !block ) continue;
         if( block->http_error_msg ) free( block->http_error_msg );
         if( block->etag ) free( block->etag );
-        if( block ) free(block);
     }
     free( transfer->block_arr );
     free( transfer->url );
@@ -285,7 +285,7 @@ static int _hbf_dav_request(hbf_transfer_t *transfer, ne_request *req, int fd, h
     const ne_status *req_status = NULL;
     const char *etag = NULL;
 
-    (void) transfer;
+    (void) transfer; // rd ???
 
     if( ! (blk && req) ) return HBF_PARAM_FAIL;
 
@@ -474,6 +474,8 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
             if( ! transfer_url ) {
                 state = HBF_PARAM_FAIL;
             }
+            DEBUG_HBF("Transfer URL: %s", transfer_url);
+
         }
 
         if( state == HBF_TRANSFER_SUCCESS ) {
@@ -540,8 +542,8 @@ Hbf_State hbf_transfer( ne_session *session, hbf_transfer_t *transfer, const cha
             } else {
                 state = HBF_MEMORY_FAIL;
             }
-            free( transfer_url );
         }
+        free( transfer_url );
     }
 
     /* do the source file validation finally (again). */
