@@ -27,7 +27,7 @@
 
 namespace Mirall {
 
-static uint chunkSize() {
+static qint64 chunkSize() {
     static uint chunkSize;
     if (!chunkSize) {
         chunkSize = qgetenv("OWNCLOUD_CHUNK_SIZE").toUInt();
@@ -73,7 +73,7 @@ void PUTFileJob::start() {
     }
 
     connect(reply(), SIGNAL(uploadProgress(qint64,qint64)), this, SIGNAL(uploadProgress(qint64,qint64)));
-    connect(reply(), SIGNAL(uploadProgress(qint64,qint64)), this, SLOT(resetTimeout()));
+    connect(this, SIGNAL(networkActivity()), account(), SIGNAL(propagatorNetworkActivity()));
 
     AbstractNetworkJob::start();
 }
@@ -119,14 +119,15 @@ void PropagateUploadFileQNAM::start()
 
 struct ChunkDevice : QIODevice {
 public:
-    QIODevice *_file;
+    QPointer<QIODevice> _file;
     qint64 _read;
     qint64 _size;
     qint64 _start;
 
     ChunkDevice(QIODevice *file,  qint64 start, qint64 size)
             : QIODevice(file), _file(file), _read(0), _size(size), _start(start) {
-        _file->seek(start);
+        _file = QPointer<QIODevice>(file);
+        _file.data()->seek(start);
     }
 
     virtual qint64 writeData(const char* , qint64 ) {
@@ -135,10 +136,15 @@ public:
     }
 
     virtual qint64 readData(char* data, qint64 maxlen) {
+        if (_file.isNull()) {
+            qDebug() << Q_FUNC_INFO << "Upload file object deleted during upload";
+            close();
+            return -1;
+        }
         maxlen = qMin(maxlen, chunkSize() - _read);
         if (maxlen == 0)
             return 0;
-        qint64 ret = _file->read(data, maxlen);
+        qint64 ret = _file.data()->read(data, maxlen);
         if (ret < 0)
             return -1;
         _read += ret;
@@ -146,7 +152,11 @@ public:
     }
 
     virtual bool atEnd() const {
-        return  _read >= chunkSize() || _file->atEnd();
+        if (_file.isNull()) {
+            qDebug() << Q_FUNC_INFO << "Upload file object deleted during upload";
+            return true;
+        }
+        return  _read >= chunkSize() || _file.data()->atEnd();
     }
 
     virtual qint64 size() const{
@@ -164,8 +174,13 @@ public:
     }
 
     virtual bool seek ( qint64 pos ) {
+        if (_file.isNull()) {
+            qDebug() << Q_FUNC_INFO << "Upload file object deleted during upload";
+            close();
+            return false;
+        }
         _read = pos;
-        return _file->seek(pos + _start);
+        return _file.data()->seek(pos + _start);
     }
 };
 
@@ -214,7 +229,7 @@ void PropagateUploadFileQNAM::startNextChunk()
                 currentChunkSize = chunkSize();
             }
         }
-        device = new ChunkDevice(_file, chunkSize() * sendingChunk, currentChunkSize);
+        device = new ChunkDevice(_file, chunkSize() * quint64(sendingChunk), currentChunkSize);
     } else {
         device = _file;
     }
@@ -287,8 +302,10 @@ void PropagateUploadFileQNAM::slotPutFinished()
             return;
         }
 
-        if (Utility::qDateTimeToTime_t(fi.lastModified()) != _item._modtime) {
-            /* Uh oh:  The local file has changed during upload */
+        const time_t new_mtime = FileSystem::getModTime(fi.absoluteFilePath());
+        if (new_mtime != _item._modtime) {
+            qDebug() << "The local file has changed during upload:" << _item._modtime << "!=" << new_mtime
+                     << ", QFileInfo: " << Utility::qDateTimeToTime_t(fi.lastModified()) << fi.lastModified();
             _propagator->_activeJobs--;
             done(SyncFileItem::SoftError, tr("Local file changed during sync."));
             // FIXME:  the legacy code was retrying for a few seconds.
@@ -380,6 +397,12 @@ void PropagateUploadFileQNAM::abort()
 ///////////////////////////////////////////////////////////////////////////////////////////////////
 
 void GETFileJob::start() {
+    if (_resumeStart > 0) {
+        _headers["Range"] = "bytes=" + QByteArray::number(_resumeStart) +'-';
+        _headers["Accept-Ranges"] = "bytes";
+        qDebug() << "Retry with range " << _headers["Range"];
+    }
+
     QNetworkRequest req;
     for(QMap<QByteArray, QByteArray>::const_iterator it = _headers.begin(); it != _headers.end(); ++it) {
         req.setRawHeader(it.key(), it.value());
@@ -387,6 +410,7 @@ void GETFileJob::start() {
 
     setReply(davRequest("GET", path(), req));
     setupConnections(reply());
+    reply()->setReadBufferSize(128 * 1024);
 
     if( reply()->error() != QNetworkReply::NoError ) {
         qWarning() << Q_FUNC_INFO << " Network error: " << reply()->errorString();
@@ -395,15 +419,22 @@ void GETFileJob::start() {
     connect(reply(), SIGNAL(metaDataChanged()), this, SLOT(slotMetaDataChanged()));
     connect(reply(), SIGNAL(readyRead()), this, SLOT(slotReadyRead()));
     connect(reply(), SIGNAL(downloadProgress(qint64,qint64)), this, SIGNAL(downloadProgress(qint64,qint64)));
+    connect(this, SIGNAL(networkActivity()), account(), SIGNAL(propagatorNetworkActivity()));
 
     AbstractNetworkJob::start();
 }
 
 void GETFileJob::slotMetaDataChanged()
 {
-    if (reply()->error() != QNetworkReply::NoError
-            || reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt() / 100 != 2) {
-        // We will handle the error when the job is finished.
+    int httpStatus = reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+    // If the status code isn't 2xx, don't write the reply body to the file.
+    // For any error: handle it when the job is finished, not here.
+    if (httpStatus / 100 != 2) {
+        _device->close();
+        return;
+    }
+    if (reply()->error() != QNetworkReply::NoError) {
         return;
     }
 
@@ -423,11 +454,39 @@ void GETFileJob::slotMetaDataChanged()
         reply()->abort();
         return;
     }
+
+    quint64 start = 0;
+    QByteArray ranges = parseEtag(reply()->rawHeader("Content-Range"));
+    if (!ranges.isEmpty()) {
+        QRegExp rx("bytes (\\d+)-");
+        if (rx.indexIn(ranges) >= 0) {
+            start = rx.cap(1).toULongLong();
+        }
+    }
+    if (start != _resumeStart) {
+        qDebug() << Q_FUNC_INFO <<  "Wrong content-range: "<< ranges << " while expecting start was" << _resumeStart;
+        if (start == 0) {
+            // device don't support range, just stry again from scratch
+            _device->close();
+            if (!_device->open(QIODevice::WriteOnly)) {
+                _errorString = _device->errorString();
+                _errorStatus = SyncFileItem::NormalError;
+                reply()->abort();
+                return;
+            }
+        } else {
+            _errorString = tr("Server returned wrong content-range");
+            _errorStatus = SyncFileItem::NormalError;
+            reply()->abort();
+            return;
+        }
+    }
+
 }
 
 void GETFileJob::slotReadyRead()
 {
-    int bufferSize = qMax(1024*8ll , reply()->bytesAvailable());
+    int bufferSize = qMin(1024*8ll , reply()->bytesAvailable());
     QByteArray buffer(bufferSize, Qt::Uninitialized);
 
     while(reply()->bytesAvailable() > 0) {
@@ -440,16 +499,17 @@ void GETFileJob::slotReadyRead()
             return;
         }
 
-        qint64 w = _device->write(buffer.constData(), r);
-        if (w != r) {
-            _errorString = _device->errorString();
-            _errorStatus = SyncFileItem::NormalError;
-            qDebug() << "Error while writing to file" << w << r <<  _errorString;
-            reply()->abort();
-            return;
+        if (_device->isOpen()) {
+            qint64 w = _device->write(buffer.constData(), r);
+            if (w != r) {
+                _errorString = _device->errorString();
+                _errorStatus = SyncFileItem::NormalError;
+                qDebug() << "Error while writing to file" << w << r <<  _errorString;
+                reply()->abort();
+                return;
+            }
         }
     }
-    resetTimeout();
 }
 
 void PropagateDownloadFileQNAM::start()
@@ -514,22 +574,18 @@ void PropagateDownloadFileQNAM::start()
     /* Allow compressed content by setting the header */
     //headers["Accept-Encoding"] = "gzip";
 
-    if (_tmpFile.size() > 0) {
-        quint64 done = _tmpFile.size();
-        if (done == _item._size) {
+    _startSize = _tmpFile.size();
+    if (_startSize > 0) {
+        if (_startSize == _item._size) {
             qDebug() << "File is already complete, no need to download";
             downloadFinished();
             return;
         }
-        headers["Range"] = "bytes=" + QByteArray::number(done) +'-';
-        headers["Accept-Ranges"] = "bytes";
-        qDebug() << "Retry with range " << headers["Range"];
-        _startSize = done;
     }
 
     _job = new GETFileJob(AccountManager::instance()->account(),
                           _propagator->_remoteFolder + _item._file,
-                          &_tmpFile, headers, expectedEtagForResume);
+                          &_tmpFile, headers, expectedEtagForResume, _startSize);
     _job->setTimeout(_propagator->httpTimeout() * 1000);
     connect(_job, SIGNAL(finishedSignal()), this, SLOT(slotGetFinished()));
     connect(_job, SIGNAL(downloadProgress(qint64,qint64)), this, SLOT(slotDownloadProgress(qint64,qint64)));
@@ -551,18 +607,33 @@ void PropagateDownloadFileQNAM::slotGetFinished()
 
     QNetworkReply::NetworkError err = job->reply()->error();
     if (err != QNetworkReply::NoError) {
-        if (_tmpFile.size() == 0) {
-            // don't keep the temporary file if it is empty.
+        _item._httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
+        // If we sent a 'Range' header and get 416 back, we want to retry
+        // without the header.
+        bool badRangeHeader = job->resumeStart() > 0 && _item._httpErrorCode == 416;
+        if (badRangeHeader) {
+            qDebug() << Q_FUNC_INFO << "server replied 416 to our range request, trying again without in next sync";
+        }
+
+        // Don't keep the temporary file if it is empty or we
+        // used a bad range header.
+        if (_tmpFile.size() == 0 || badRangeHeader) {
             _tmpFile.close();
             _tmpFile.remove();
             _propagator->_journal->setDownloadInfo(_item._file, SyncJournalDb::DownloadInfo());
         }
-        _item._httpErrorCode = job->reply()->attribute(QNetworkRequest::HttpStatusCodeAttribute).toInt();
+
         _propagator->_activeJobs--;
         SyncFileItem::Status status = job->errorStatus();
         if (status == SyncFileItem::NoStatus) {
             status = classifyError(err, _item._httpErrorCode);
         }
+        if (badRangeHeader) {
+            // Can't do this in classifyError() because 416 without a
+            // Range header should result in NormalError.
+            status = SyncFileItem::SoftError;
+        }
         done(status, job->errorString());
         return;
     }
