@@ -22,30 +22,45 @@
 
 #include <neon/ne_socket.h>
 
-#include "csyncthread.h"
-#include <syncjournaldb.h>
-#include "logger.h"
+#include "mirall/syncengine.h"
+#include "mirall/syncjournaldb.h"
+#include "mirall/logger.h"
 #include "csync.h"
 #include "mirall/clientproxy.h"
+#include "mirall/account.h"
+#include "creds/httpcredentials.h"
+#include "owncloudcmd.h"
+#include "simplesslerrorhandler.h"
 
 using namespace Mirall;
 
-int getauth(const char* prompt, char* buf, size_t len, int echo, int verify, void*)
-{
-    std::cout << "** Authentication required: \n" << prompt << std::endl;
-    std::string s;
-    std::getline(std::cin, s);
-    strncpy( buf, s.c_str(), len );
-    return 0;
-}
-
 struct CmdOptions {
     QString source_dir;
     QString target_url;
     QString config_directory;
     QString proxy;
+    bool silent;
+    bool trustSSL;
 };
 
+int getauth(const char* prompt, char* buf, size_t len, int a, int b, void *userdata)
+{
+    (void) a;
+    (void) b;
+
+    struct CmdOptions *opts = (struct CmdOptions*) userdata;
+
+    std::cout << "** Authentication required: \n" << prompt << std::endl;
+    std::string s;
+    if(opts && opts->trustSSL) {
+        s = "yes";
+    } else {
+        std::getline(std::cin, s);
+    }
+    strncpy( buf, s.c_str(), len );
+    return 0;
+}
+
 void help()
 {
     std::cout << "owncloudcmd - command line ownCloud client tool." << std::endl;
@@ -56,9 +71,11 @@ void help()
     std::cout << "uses the setting from a configured sync client." << std::endl;
     std::cout << std::endl;
     std::cout << "Options:" << std::endl;
+    std::cout << "  --silent               Don't be so verbose" << std::endl;
     std::cout << "  --confdir = configdir: Read config from there." << std::endl;
     std::cout << "  --httpproxy = proxy:   Specify a http proxy to use." << std::endl;
     std::cout << "                         Proxy is http://server:port" << std::endl;
+    std::cout << "  --trust                Trust the SSL certification." << std::endl;
     std::cout << "" << std::endl;
     exit(1);
 
@@ -80,9 +97,11 @@ void parseOptions( const QStringList& app_args, CmdOptions *options )
         }
         options->target_url.append("remote.php/webdav/");
     }
+    if (options->target_url.startsWith("http"))
+        options->target_url.replace(0, 4, "owncloud");
     options->source_dir = args.takeLast();
     if( !QFile::exists( options->source_dir )) {
-        std::cerr << "Source dir does not exists.";
+        std::cerr << "Source dir does not exists." << std::endl;
         exit(1);
     }
 
@@ -97,6 +116,10 @@ void parseOptions( const QStringList& app_args, CmdOptions *options )
             options->config_directory = it.next();
         } else if( option == "--httpproxy" && !it.peekNext().startsWith("-")) {
             options->proxy = it.next();
+        } else if( option == "--silent") {
+            options->silent = true;
+        } else if( option == "--trust") {
+            options->trustSSL = true;
         } else {
             help();
         }
@@ -110,11 +133,35 @@ void parseOptions( const QStringList& app_args, CmdOptions *options )
 int main(int argc, char **argv) {
     QCoreApplication app(argc, argv);
 
+
+
     CmdOptions options;
+    options.silent = false;
+    options.trustSSL = false;
     ClientProxy clientProxy;
 
     parseOptions( app.arguments(), &options );
 
+    QSslConfiguration config = QSslConfiguration::defaultConfiguration();
+    config.setProtocol(QSsl::TlsV1);
+    QSslConfiguration::setDefaultConfiguration(config);
+
+    QUrl url(options.target_url.toUtf8());
+    Account account;
+
+    // Find the folder and the original owncloud url
+    QStringList splitted = url.path().split(account.davPath());
+    url.setPath(splitted.value(0));
+    url.setScheme(url.scheme().replace("owncloud", "http"));
+    QString folder = splitted.value(1);
+
+    SimpleSslErrorHandler *sslErrorHandler = new SimpleSslErrorHandler;
+
+    account.setUrl(url);
+    account.setCredentials(new HttpCredentials(url.userName(), url.password()));
+    account.setSslErrorHandler(sslErrorHandler);
+    AccountManager::instance()->setAccount(&account);
+
     CSYNC *_csync_ctx;
     if( csync_create( &_csync_ctx, options.source_dir.toUtf8(),
                       options.target_url.toUtf8()) < 0 ) {
@@ -126,16 +173,15 @@ int main(int argc, char **argv) {
         qFatal("ne_sock_init failed!");
     }
 
-    csync_set_log_level(1);
-    csync_enable_conflictcopys(_csync_ctx);
+    csync_set_log_level(options.silent ? 1 : 11);
     Logger::instance()->setLogFile("-");
 
+    csync_set_userdata(_csync_ctx, &options);
     csync_set_auth_callback( _csync_ctx, getauth );
 
     if( csync_init( _csync_ctx ) < 0 ) {
         qFatal("Could not initialize csync!");
         return EXIT_FAILURE;
-        _csync_ctx = 0;
     }
 
     csync_set_module_property(_csync_ctx, "csync_context", _csync_ctx);
@@ -174,10 +220,15 @@ int main(int argc, char **argv) {
         clientProxy.setCSyncProxy(QUrl(url), _csync_ctx);
     }
 
+    OwncloudCmd owncloudCmd;
+
     SyncJournalDb db(options.source_dir);
-    CSyncThread csyncthread(_csync_ctx, options.source_dir, QUrl(options.target_url).path(), &db);
-    QObject::connect(&csyncthread, SIGNAL(finished()), &app, SLOT(quit()));
-    csyncthread.startSync();
+    SyncEngine engine(_csync_ctx, options.source_dir, QUrl(options.target_url).path(), folder, &db);
+    QObject::connect(&engine, SIGNAL(finished()), &app, SLOT(quit()));
+    QObject::connect(&engine, SIGNAL(transmissionProgress(Progress::Info)), &owncloudCmd, SLOT(transmissionProgressSlot()));
+
+    // Have to be done async, else, an error before exec() does not terminate the event loop.
+    QMetaObject::invokeMethod(&engine, "startSync", Qt::QueuedConnection);
 
     app.exec();
 
@@ -187,3 +238,4 @@ int main(int argc, char **argv) {
 
     return 0;
 }
+
